@@ -367,9 +367,9 @@ class Post(KnockerModel, BlogMetaMixin, TranslatableModel):
                 kwargs["month"] = "%02d" % current_date.month
             if "<int:day>" in urlconf:
                 kwargs["day"] = "%02d" % current_date.day
-            if "<slug:slug>" in urlconf:
+            if "<str:slug>" in urlconf or "<slug:slug>" in urlconf:
                 kwargs["slug"] = self.safe_translation_getter("slug", language_code=lang, any_language=True)  # NOQA
-            if "<slug:category>" in urlconf:
+            if "<slug:category>" in urlconf or "<str:category>" in urlconf:
                 kwargs["category"] = category.safe_translation_getter(
                     "slug", language_code=lang, any_language=True
                 )  # NOQA
